/*
 * Portions of this software are copyrighted by the 
 * 
 *     University of South Carolina,
 *     Sun Microsystems, Inc., 
 * 
 * and other parties. 
 * 
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
 * DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
 * IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
 * NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */


package edu.sc.seis.TauP;

import java.io.*;
import java.awt.*;
import java.util.*;

/**
  * Simple polar plot widget.
  *
  * @version 1.0  Tue Feb  2 20:48:34 GMT 1999
  * @author H. Philip Crotwell
  */
public class PolarPlot extends Canvas {

   Container pappy;

   Dimension minSize;

   int width;   // circles, so height=width

   protected Vector segments = new Vector();
   
   double[] circleRadius;
   
   double outsideRadius = 6371.0;
   
   int centerX, centerY;

	public static final short FULL = 0;
	public static final short HALF = 1;
	public static final short QUARTER = 2;
	short displayMode;
		
   public PolarPlot(Container parent) {
      pappy = parent;
      width = 250;
      minSize = new Dimension(width,width);
   	setDisplayMode(FULL);
   }

   public PolarPlot(Container parent, int width) {
      pappy = parent;
      this.width = width;
      minSize = new Dimension(width,width);
   	setDisplayMode(FULL);

   }

	public void setCircles(double[] circles) {
		circleRadius = new double[circles.length];
		for (int i=0;i<circles.length;i++) {
			circleRadius[i] = circles[i];
		}
	}
	
	public void setOutsideRadius(double r) {
		outsideRadius = r;
	}
	
	public void appendSegment(TimeDist[] td) {
		segments.addElement(td);
	}
	
	public void clearSegments() {
		segments.removeAllElements();
	}
	
	public void setDisplayMode(short mode) {
		displayMode = mode;
		switch (displayMode) {
			case FULL:
				centerX = width/2;
				centerY = width/2;
				break;
			case HALF:
				centerX = width/2;
				centerY = width;
				break;
			case QUARTER:
				centerX = 0;
				centerY = width;
				break;
		}
	}
	
   public Dimension preferredSize() {
      return minimumSize();
   }

   public synchronized Dimension minimumSize() {
      return minSize;
   }
   
   public Insets insets() {
      return new Insets(5,5,5,5);
   }
 
   public void paint(Graphics g) {
      FontMetrics fontMetrics = g.getFontMetrics();

      g.drawRect(0, 0, width-1, width-1);
		plotCircles(g);
      plotData(g);
   }

	protected void plotCircles(Graphics g) {
		int radius;
		if (circleRadius != null) {
			switch (displayMode) {
				case FULL:
					for (int i=0; i<circleRadius.length; i++) {
						radius = (int)Math.rint(circleRadius[i]/outsideRadius*width/2);
						g.drawOval(width/2-radius, width/2-radius,
							 2*radius, 2*radius);
					}
					break;
				case HALF:
					for (int i=0; i<circleRadius.length; i++) {
						radius = (int)Math.rint(circleRadius[i]/outsideRadius*width/2);
						g.drawArc(width/2-radius, width-radius,
							2*radius, 2*radius, 0, 180);
					}
					break;
				case QUARTER:
					for (int i=0; i<circleRadius.length; i++) {
						radius = (int)Math.rint(circleRadius[i]/outsideRadius*width);
						g.drawArc(-1*radius, width-radius,
							2*radius, 2*radius, 0, 90);
					}
					break;
			}
		}
	}

   protected void plotData(Graphics g) {
		int numSegments = segments.size();
		double scale = centerY/outsideRadius;
      for (int segNum=0;segNum<numSegments;segNum++) {
         g.setColor(colorForSegment(segNum));
         TimeDist[] data = (TimeDist[])segments.elementAt(segNum);
         for (int i=0;i<data.length-1;i++) {
            g.drawLine(centerX - (int)Math.rint(
            	(outsideRadius-data[i].depth)*Math.cos(data[i].dist+Math.PI/2)*scale), 
               centerY - (int)Math.rint(
            	(outsideRadius-data[i].depth)*Math.sin(data[i].dist+Math.PI/2)*scale),
               centerX - (int)Math.rint(
            	(outsideRadius-data[i+1].depth)*Math.cos(data[i+1].dist+Math.PI/2)*scale), 
               centerY - (int)Math.rint(
            	(outsideRadius-data[i+1].depth)*Math.sin(data[i+1].dist+Math.PI/2)*scale));
         }
      }
      g.setColor(Color.black);
   }

   public Color colorForSegment(int segNum) {
      if (segments.size() == 1) return Color.black;

      int colorInc = (int)Math.floor(255/(segments.size()-1));
      return new Color(segNum*colorInc,255-segNum*colorInc,(segNum % 3)*127);
   }

}
